import {
  select,
  scaleLinear,
  scaleOrdinal,
  scaleSqrt,
  extent,
  min,
  max,
  axisLeft,
  axisBottom,
  symbols,
  symbol
} from 'd3';

export const scatterPlot = () => {
  let width = 960;
  let height = 500;
  let data = [];
  let xValue = [];
  let yValue = [];
  let symbolValue = [];
  let margin = {
    top: 10,
    right: 20,
    bottom: 10,
    left: 20,
  };
  let size = 5;

  const my = (selection) => {
    const xScale = scaleLinear()
      .domain(
        extent(data, xValue) // [min(data, xValue), max(data, xValue)]
      )
      .range([margin.left, width - margin.right]);

    const yScale = scaleLinear()
      .domain([min(data, yValue), max(data, yValue)])
      .range([
        height - margin.bottom,
        margin.top,
      ]);
    
    const symbolScale = scaleOrdinal()
    	.domain(data.map(symbolValue))
    	.range(symbols);
    
    
    const symbolGenerator = symbol().size(size)

    const marks = data.map((d) => ({
      x: xScale(xValue(d)),
      y: yScale(yValue(d)),
      pathData: symbolGenerator.type(symbolScale(symbolValue(d)))(),
    }));

    selection
      .selectAll('path')
      .data(marks)
      .join('path')
      .attr('d', d => d.pathData)
    	.attr('transform', d => `translate(${d.x}, ${d.y})`);

    selection
      .append('g')
      .attr(
        'transform',
        `translate(${0}, ${
          height - margin.bottom
        })`
      )
      .call(axisBottom(xScale));

    selection
      .append('g')
      .attr(
        'transform',
        `translate(${margin.left}, ${0})`
      )
      .call(axisLeft(yScale));
  };

  my.width = function (_) {
    return arguments.length
      ? ((width = +_), my)
      : width;
  };

  my.height = function (_) {
    return arguments.length
      ? ((height = +_), my)
      : height;
  };

  my.data = function (_) {
    return arguments.length
      ? ((data = _), my)
      : data;
  };

  my.xValue = function (_) {
    return arguments.length
      ? ((xValue = _), my)
      : xValue;
  };

  my.yValue = function (_) {
    return arguments.length
      ? ((yValue = _), my)
      : yValue;
  };
  
  my.symbolValue = function (_) {
    return arguments.length
      ? ((symbolValue = _), my)
      : symbolValue;
  };

  my.margin = function (_) {
    return arguments.length
      ? ((margin = _), my)
      : margin;
  };

  my.size = function (_) {
    return arguments.length
      ? ((size = +_), my)
      : size;
  };

  return my;
};
